@@ -6,7 +6,7 @@ use warnings;
 use Algorithm::Combinatorics qw(subsets variations);
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.14;
+our $VERSION = 0.15;
 
 my$self;
 
@@ -32,14 +32,17 @@ sub new{
 			minJaccard => undef
 		},
 		output => undef,
+		pathOutputFiles => "",
 		messages => undef,
 		keyItemsDescription => undef,
 		keyItemsTransactions => undef,
 		limitRules => undef,
+		limitSubsets => undef,
+		numberSubsets => 0,
 		frequentItemset => [],
 		associationRules => undef,
 		implications => undef,
-		largeItemsetSize => 2,
+		largeItemsetLength => 2,
 		rule => 0
 	};
 	bless($self,$class);
@@ -97,31 +100,34 @@ sub generate_rules{
 	if($self->{messages}){
 		print "\n${\scalar(keys(%{$self->{keyItemsDescription}}))} items, ${\$self->quantity_possible_rules} possible rules";
 	}
-	my @largeItemsetSizeOne = grep{(($self->{keyItemsTransactions}{$_}/$self->{totalTransactions})*100)>=$self->{metrics}{minSupport}}keys(%{$self->{keyItemsDescription}});
-	$self->association_rules(\@largeItemsetSizeOne);
+	my @largeItemsetLengthOne = grep{(($self->{keyItemsTransactions}{$_}/$self->{totalTransactions})*100)>=$self->{metrics}{minSupport}}keys(%{$self->{keyItemsDescription}});
+	$self->association_rules(\@largeItemsetLengthOne);
 }
 
 sub association_rules{
 	my @largeItemset = @{$_[1]};
-	my @variations = variations(\@largeItemset,$self->{largeItemsetSize});
+	my @variations = variations(\@largeItemset,$self->{largeItemsetLength});
 	my @frequentItemset;
 	if($self->{messages}){
-		print "\nLarge itemset size $self->{largeItemsetSize}, ${\scalar(@largeItemset)} items ";
+		print "\nLarge itemset of length $self->{largeItemsetLength}, ${\scalar(@largeItemset)} items ";
 		print "\nProcessing... ";
 	}
+	VARIATIONS:
 	foreach my$variation(@variations){
 		my@candidateItemset=@{$variation};
 		my@antecedent;
 		my@consequent;
-		for(my$antecedentSize=0;$antecedentSize<$#candidateItemset;$antecedentSize++){
-			push@antecedent,$candidateItemset[$antecedentSize];
+		for(my$antecedentLength=0;$antecedentLength<$#candidateItemset;$antecedentLength++){
+			push@antecedent,$candidateItemset[$antecedentLength];
 			@consequent=();
-			for(my$consequentSize=($antecedentSize+1);$consequentSize<=$#candidateItemset;$consequentSize++){
-				push@consequent,$candidateItemset[$consequentSize];
+			for(my$consequentLength=($antecedentLength+1);$consequentLength<=$#candidateItemset;$consequentLength++){
+				push@consequent,$candidateItemset[$consequentLength];
 			}
 			@antecedent=sort@antecedent;
 			@consequent=sort@consequent;
 			next if("@consequent"~~@{$self->{implications}{"@antecedent"}});
+			last VARIATIONS if(defined $self->{limitSubsets} && $self->{numberSubsets} == $self->{limitSubsets});
+			$self->{numberSubsets}++;
 			push @{$self->{implications}{"@antecedent"}},"@consequent";
 			my@implication;
 			push@implication,@antecedent,@consequent;
@@ -177,8 +183,8 @@ sub association_rules{
 			};
 			my@items=grep{!($_~~@frequentItemset)}@implication;
 			push@frequentItemset,@items;
+			last VARIATIONS if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
 		}
-		last if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
 	}
 	if($self->{messages}){
 		print "\nFrequent itemset: { @frequentItemset }, ${\scalar(@frequentItemset)} items ";
@@ -187,15 +193,17 @@ sub association_rules{
 		@{$self->{frequentItemset}}=@frequentItemset;
 		$self->output;
 	}
-	return if(defined $self->{limitRules} && $self->{rule} == $self->{limitRules});
-	if(scalar(@frequentItemset)>=($self->{largeItemsetSize}+1)){
-		$self->{largeItemsetSize}++;
+	return if((defined $self->{limitRules} && $self->{rule} == $self->{limitRules})
+				||(defined $self->{limitSubsets} && $self->{numberSubsets} == $self->{limitSubsets}));
+	if(scalar(@frequentItemset)>=($self->{largeItemsetLength}+1)){
+		$self->{largeItemsetLength}++;
 		$self->{associationRules} = undef;
 		$self->association_rules(\@frequentItemset);
 	}
 }
 
 sub stop{
+	print "$self->{rule} - $self->{numberSubsets}";
 	if($self->{messages}){
 		print "\nStopping... ";
 		$self->output if $self->{associationRules};
@@ -228,10 +236,10 @@ sub output{
 
 sub file{
 	if($self->{messages}){
-		print "\nExporting to file \"output_large_itemset_size_$self->{largeItemsetSize}.txt\"... ";
+		print "\nExporting to file $self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.txt... ";
 	}
-	open(FILE,">output_large_itemset_size_$self->{largeItemsetSize}.txt")
-		or die("\nError: \"output_large_itemset_size_$self->{largeItemsetSize}.txt\" $!");
+	open(FILE,">$self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.txt")
+		or die("\nError: $self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.txt $!");
 	print FILE "Rules\tSupport\tConfidence";
 	my@headings=('Lift', 'Leverage', 'Conviction', 'Coverage', 'Correlation', 'Cosine', 'Laplace', 'Jaccard');
 	my@metrics;
@@ -279,10 +287,10 @@ sub file{
 sub excel{
 	require Excel::Writer::XLSX;
 	if($self->{messages}){
-		print "\nExporting to excel \"output_large_itemset_size_$self->{largeItemsetSize}.xlsx\"... ";
+		print "\nExporting to excel $self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.xlsx... ";
 	}
-	my $workbook  = Excel::Writer::XLSX->new("output_large_itemset_size_$self->{largeItemsetSize}.xlsx") 
-		or die("\nError: \"output_large_itemset_size_$self->{largeItemsetSize}.xlsx\" $!");
+	my $workbook  = Excel::Writer::XLSX->new("$self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.xlsx") 
+		or die("\nError: $self->{pathOutputFiles}output_large_itemset_length_$self->{largeItemsetLength}.xlsx $!");
 	my $worksheet = $workbook->add_worksheet();
 	my $bold = $workbook->add_format(bold => 1);
 	my $headings = ['Rules', 'Support', 'Confidence'];
@@ -341,7 +349,14 @@ sub excel{
 	}
 	$worksheet->insert_chart($columns[($i+2)].'2', $chart);
 	$line+=2;
+	my $urlFormat = $workbook->add_format(
+	    color     => 'blue',
+	);
+	my$url=2;
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
+		$worksheet->write_url("A$line","internal:Sheet1!A$url",$urlFormat,"<");
+		$line++;
+		$worksheet->write_url("A$url","internal:Sheet1!A$line",$urlFormat,"R$rule");
 		$worksheet->write("A$line","Rule R$rule: $self->{associationRules}{$rule}{implication}");
 		$line++;
 		$worksheet->write("A$line","Support: $self->{associationRules}{$rule}{support}");
@@ -359,6 +374,7 @@ sub excel{
 			$line++;
 		}
 		$line++;
+		$url++;
 	}
 	$worksheet->write("A$line","Frequent itemset: { @{$self->{frequentItemset}} }");
 	$line++;
@@ -420,6 +436,8 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	$apriori->{output}=1;
 	# The output type(optional): 1 - Export to text file delimited by tab; 2 - Export to excel file with chart.
 
+	$apriori->{pathOutputFiles}='data/'; # The path to output files(optional)
+
 	$apriori->{messages}=1; # A value boolean to display the messages(optional)
 
 	$apriori->{keyItemsDescription}{'101'}='MILK'; # Hash table reference to add items by key and description
@@ -458,7 +476,9 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	print "\n${\$apriori->quantity_possible_rules}"; # Show the quantity of possible rules
 
-	$apriori->{limitRules}=10; # The limit of rules
+	$apriori->{limitRules}=10; # The limit of rules(optional)
+
+	$apriori->{limitSubsets}=12; # The limit of subsets(optional)
 
 	$apriori->generate_rules;
 	# Generate association rules to no longer meet the minimum support, confidence, lift, leverage, conviction, coverage, correlation, cosine, laplace, jaccard or limit of rules
@@ -469,17 +489,17 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	12
 	3 items, 12 possible rules
-	Large itemset size 2, 3 items
+	Large itemset of length 2, 3 items
 	Processing...
-	Frequent itemset: { 103, 102, 101 }, 3 items
-	Exporting to excel "output_large_itemset_size_2.xlsx"...
-	Large itemset size 3, 3 items
+	Frequent itemset: { 101, 103, 102 }, 3 items
+	Exporting to file data/output_large_itemset_length_2.txt...
+	Large itemset of length 3, 3 items
 	Processing...
-	Frequent itemset: { 103, 101, 102 }, 3 items
-	Exporting to excel "output_large_itemset_size_3.xlsx"...
-	103, 101, 102
+	Frequent itemset: { 101, 102, 103 }, 3 items
+	Exporting to file data/output_large_itemset_length_3.txt...
+	101, 102, 103
 
-	#output file "output_itemset_size_2.txt"
+	#output file "output_itemset_length_2.txt"
 
 	Rules	Support	Confidence	Lift	Leverage	Conviction	Coverage	Correlation	Cosine	Laplace	Jaccard
 	R1	0,7000	0,8750	1,2500	0,1400	2,4000	0,8000	0,7638	0,9354	0,6071	0,8750
@@ -506,7 +526,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	to be continued...
 
-	#output file "output_itemset_size_3.txt"
+	#output file "output_itemset_length_3.txt"
 
 	Rules	Support	Confidence	Lift	Leverage	Conviction	Coverage	Correlation	Cosine	Laplace	Jaccard
 	R7	0,6000	0,7500	1,2500	0,1200	1,6000	0,8000	0,6124	0,8660	0,5714	0,7500
@@ -628,6 +648,8 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{output}=1;
 
+	$apriori->{pathOutputFiles}='data/';
+
 	$apriori->{messages}=1;
 
 	my $db = DBI->connect('dbi:SQLite:dbname=DW.db','','');
@@ -667,6 +689,8 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{limitRules}=10;
 
+	$apriori->{limitSubsets}=12;
+
 	$apriori->generate_rules;
 
 	print "\n@{$apriori->{frequentItemset}}\n";
@@ -733,6 +757,10 @@ The minimum jaccard(optional)
 
 The limit of rules(optional)
 
+=head2 limitSubsets
+
+The limit of subsets(optional)
+
 =head2 output
 
 The output type(optional):
@@ -749,6 +777,10 @@ The output type(optional):
 
 =back
 
+=head2 pathOutputFiles
+
+The path to output files(optional)
+
 =head2 messages
 
 A value boolean to display the messages(optional)
@@ -846,7 +878,7 @@ Generate association rules until no set of items meets the minimum support, conf
 
 =head2 association_rules
 
-Generate association rules by size of large itemsets.
+Generate association rules by length of large itemsets.
 
 =head1 AUTHOR
 
